(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rxjs'), require('rxjs/operators'), require('@ng-web-apis/common'), require('@angular/core')) :
    typeof define === 'function' && define.amd ? define('@ng-web-apis/midi', ['exports', 'rxjs', 'rxjs/operators', '@ng-web-apis/common', '@angular/core'], factory) :
    (factory((global['ng-web-apis'] = global['ng-web-apis'] || {}, global['ng-web-apis'].midi = {}),global.rxjs,global.rxjs.operators,global.common,global.ng.core));
}(this, (function (exports,rxjs,operators,common,core) { 'use strict';

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * @param {?} value
     * @param {?} min
     * @param {?} max
     * @return {?}
     */
    function between(value, min, max) {
        return value >= min && value <= max;
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Filter MIDI messages to aftertouch changes only
     * @return {?}
     */
    function aftertouch() {
        return function (source) {
            return source.pipe(operators.filter(function (_a) {
                var data = _a.data;
                return between(data[0], 208, 223);
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Filter MIDI messages by channel
     *
     * @param {?} channel number from 0 to 15
     * @return {?}
     */
    function filterByChannel(channel) {
        return function (source) {
            return source.pipe(operators.filter(function (_a) {
                var data = _a.data;
                return data[0] % 16 === channel;
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Filter MIDI messages by MIDIInput id
     *
     * @param {?} id
     * @return {?}
     */
    function filterById(id) {
        return function (source) {
            return source.pipe(operators.filter(function (_a) {
                var target = _a.target;
                return (( /** @type {?} */(target))).id === id;
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Filter MIDI messages by MIDIInput name
     *
     * @param {?} name
     * @return {?}
     */
    function filterByName(name) {
        return function (source) {
            return source.pipe(operators.filter(function (_a) {
                var target = _a.target;
                return (( /** @type {?} */(target))).name === name;
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Filter MIDI messages to main volume changes only
     * @return {?}
     */
    function mainVolume() {
        return function (source) {
            return source.pipe(operators.filter(function (_a) {
                var data = _a.data;
                return between(data[0], 176, 191) && data[1] === 7;
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Filter MIDI messages to modulation wheel changes only
     * @return {?}
     */
    function modulationWheel() {
        return function (source) {
            return source.pipe(operators.filter(function (_a) {
                var data = _a.data;
                return between(data[0], 176, 191) && data[1] === 1;
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Filter MIDI messages to notes only
     *
     * IMPORTANT: It normalizes noteOff events to noteOn with 0 velocity
     * @return {?}
     */
    function notes() {
        return function (source) {
            return source.pipe(operators.filter(function (_a) {
                var data = _a.data;
                return between(data[0], 128, 159);
            }), operators.map(function (event) {
                if (between(event.data[0], 128, 143)) {
                    event.data[0] += 16;
                    event.data[2] = 0;
                }
                return event;
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Filter MIDI messages to pan changes only
     * @return {?}
     */
    function pan() {
        return function (source) {
            return source.pipe(operators.filter(function (_a) {
                var data = _a.data;
                return between(data[0], 176, 191) && data[1] === 10;
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Filter MIDI messages to pitch bend changes only
     * @return {?}
     */
    function pitchBend() {
        return function (source) {
            return source.pipe(operators.filter(function (_a) {
                var data = _a.data;
                return between(data[0], 224, 239);
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Filter MIDI messages to polyphonic aftertouch changes only
     * @return {?}
     */
    function polyphonicAftertouch() {
        return function (source) {
            return source.pipe(operators.filter(function (_a) {
                var data = _a.data;
                return between(data[0], 160, 175);
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Filter MIDI messages to program changes only
     * @return {?}
     */
    function programChange() {
        return function (source) {
            return source.pipe(operators.filter(function (_a) {
                var data = _a.data;
                return between(data[0], 208, 223);
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Filter MIDI messages to sustain pedal changes only
     * @return {?}
     */
    function sustainPedal() {
        return function (source) {
            return source.pipe(operators.filter(function (_a) {
                var data = _a.data;
                return between(data[0], 176, 191) && data[1] === 64;
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Extract MIDI data from event
     * @return {?}
     */
    function toData() {
        return function (source) {
            return source.pipe(operators.map(function (_a) {
                var data = _a.data;
                return data;
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Extract data byte (2nd) from MIDI message
     *
     * NOTE: Some status messages do not have 2nd byte, use it when you're certain
     * @return {?}
     */
    function toDataByte() {
        return function (source) {
            return source.pipe(operators.map(function (_a) {
                var data = _a.data;
                return data[1];
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Extract status byte (1st) from MIDI message
     * @return {?}
     */
    function toStatusByte() {
        return function (source) {
            return source.pipe(operators.map(function (_a) {
                var data = _a.data;
                return data[0];
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Extract received time from MIDI event
     * @return {?}
     */
    function toTimeStamp() {
        return function (source) {
            return source.pipe(operators.map(function (_a) {
                var timeStamp = _a.timeStamp;
                return timeStamp;
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Extract value byte (3rd) from MIDI message
     *
     * NOTE: Some status messages do not have 3rd byte, use it when you're certain
     * @return {?}
     */
    function toValueByte() {
        return function (source) {
            return source.pipe(operators.map(function (_a) {
                var data = _a.data;
                return data[2];
            }));
        };
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /** @type {?} */
    var SYSEX = new core.InjectionToken('Require sysex MIDI access', {
        providedIn: 'root',
        factory: function () { return false; },
    });

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /** @type {?} */
    var MIDI_ACCESS = new core.InjectionToken('Promise for MIDIAccess object', {
        providedIn: 'root',
        factory: function () {
            /** @type {?} */
            var navigatorRef = core.inject(common.NAVIGATOR);
            /** @type {?} */
            var sysex = core.inject(SYSEX);
            return navigatorRef.requestMIDIAccess({ sysex: sysex });
        },
    });

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /** @type {?} */
    var MIDI_INPUT = new core.InjectionToken('MIDIInput object');

    /*! *****************************************************************************
    Copyright (c) Microsoft Corporation. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License"); you may not use
    this file except in compliance with the License. You may obtain a copy of the
    License at http://www.apache.org/licenses/LICENSE-2.0

    THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
    KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
    WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
    MERCHANTABLITY OR NON-INFRINGEMENT.

    See the Apache Version 2.0 License for specific language governing permissions
    and limitations under the License.
    ***************************************************************************** */
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /** @type {?} */
    var MIDI_MESSAGES = new core.InjectionToken('All incoming MIDI messages stream', {
        providedIn: 'root',
        factory: function () {
            return rxjs.from(core.inject(MIDI_ACCESS)).pipe(operators.switchMap(function (access) {
                return rxjs.merge.apply(void 0, __spread(Array.from(access.inputs).map(function (_a) {
                    var _b = __read(_a, 2), _ = _b[0], input = _b[1];
                    return rxjs.fromEvent(( /** @type {?} */(input)), 'midimessage');
                })));
            }));
        },
    });

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /** @type {?} */
    var MIDI_OUTPUT = new core.InjectionToken('MIDIOutput object');

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /** @type {?} */
    var MIDI_SUPPORT = new core.InjectionToken('Web MIDI API support', {
        factory: function () { return !!core.inject(common.NAVIGATOR).requestMIDIAccess; },
    });

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /** @type {?} */
    var MIDI_INPUT_QUERY = new core.InjectionToken('MIDIInput object id or name');

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Provide MIDIInput by id
     *
     * @param {?} id
     * @return {?}
     */
    function inputById(id) {
        return [
            {
                provide: MIDI_INPUT_QUERY,
                useValue: id,
            },
            {
                provide: MIDI_INPUT,
                deps: [MIDI_ACCESS, MIDI_INPUT_QUERY],
                useFactory: inputByIdFactory,
            },
        ];
    }
    /**
     * @param {?} midiAccess
     * @param {?} id
     * @return {?}
     */
    function inputByIdFactory(midiAccess, id) {
        return midiAccess.then(function (access) { return access.inputs.get(id); });
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Provide MIDIInput by name
     *
     * @param {?} name
     * @return {?}
     */
    function inputByName(name) {
        return [
            {
                provide: MIDI_INPUT_QUERY,
                useValue: name,
            },
            {
                provide: MIDI_INPUT,
                deps: [MIDI_ACCESS, MIDI_INPUT_QUERY],
                useFactory: inputByNameFactory,
            },
        ];
    }
    /**
     * @param {?} midiAccess
     * @param {?} name
     * @return {?}
     */
    function inputByNameFactory(midiAccess, name) {
        return midiAccess.then(function (access) {
            return __spread(access.inputs.values()).find(function (input) { return input.name === name; });
        });
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /** @type {?} */
    var MIDI_OUTPUT_QUERY = new core.InjectionToken('MIDIOutput object id or name');

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Provide MIDIOutput by id
     *
     * @param {?} id
     * @return {?}
     */
    function outputById(id) {
        return [
            {
                provide: MIDI_OUTPUT_QUERY,
                useValue: id,
            },
            {
                provide: MIDI_OUTPUT,
                deps: [MIDI_ACCESS, MIDI_OUTPUT_QUERY],
                useFactory: outputByIdFactory,
            },
        ];
    }
    /**
     * @param {?} midiAccess
     * @param {?} id
     * @return {?}
     */
    function outputByIdFactory(midiAccess, id) {
        return midiAccess.then(function (access) { return access.outputs.get(id); });
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Provide MIDIOutput by name
     *
     * @param {?} name
     * @return {?}
     */
    function outputByName(name) {
        return [
            {
                provide: MIDI_OUTPUT_QUERY,
                useValue: name,
            },
            {
                provide: MIDI_OUTPUT,
                deps: [MIDI_ACCESS, MIDI_OUTPUT_QUERY],
                useFactory: outputByNameFactory,
            },
        ];
    }
    /**
     * @param {?} midiAccess
     * @param {?} name
     * @return {?}
     */
    function outputByNameFactory(midiAccess, name) {
        return midiAccess.then(function (access) {
            return __spread(access.outputs.values()).find(function (output) { return output.name === name; });
        });
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /**
     * Convert MIDI notes to frequencies
     *
     * @param {?} note MIDI note
     * @param {?=} tuning tuning for middle A (440 by default)
     * @return {?}
     */
    function toFrequency(note, tuning) {
        if (tuning === void 0) {
            tuning = 440;
        }
        return Math.pow(2, (note - 69) / 12) * tuning;
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */
    /** @type {?} */
    var COEFFICIENT = Math.pow(2, (1 / 12));
    /**
     * Convert frequencies to MIDI notes
     *
     * @param {?} frequency
     * @param {?=} tuning tuning for middle A (440 by default)
     * @return {?}
     */
    function toNote(frequency, tuning) {
        if (tuning === void 0) {
            tuning = 440;
        }
        return Math.round(Math.log(frequency / tuning) / Math.log(COEFFICIENT)) + 69;
    }

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */

    /**
     * @fileoverview added by tsickle
     * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
     */

    exports.aftertouch = aftertouch;
    exports.filterByChannel = filterByChannel;
    exports.filterById = filterById;
    exports.filterByName = filterByName;
    exports.mainVolume = mainVolume;
    exports.modulationWheel = modulationWheel;
    exports.notes = notes;
    exports.pan = pan;
    exports.pitchBend = pitchBend;
    exports.polyphonicAftertouch = polyphonicAftertouch;
    exports.programChange = programChange;
    exports.sustainPedal = sustainPedal;
    exports.toData = toData;
    exports.toDataByte = toDataByte;
    exports.toStatusByte = toStatusByte;
    exports.toTimeStamp = toTimeStamp;
    exports.toValueByte = toValueByte;
    exports.MIDI_ACCESS = MIDI_ACCESS;
    exports.MIDI_INPUT = MIDI_INPUT;
    exports.MIDI_MESSAGES = MIDI_MESSAGES;
    exports.MIDI_OUTPUT = MIDI_OUTPUT;
    exports.MIDI_SUPPORT = MIDI_SUPPORT;
    exports.SYSEX = SYSEX;
    exports.inputById = inputById;
    exports.inputByIdFactory = inputByIdFactory;
    exports.inputByName = inputByName;
    exports.inputByNameFactory = inputByNameFactory;
    exports.outputById = outputById;
    exports.outputByIdFactory = outputByIdFactory;
    exports.outputByName = outputByName;
    exports.outputByNameFactory = outputByNameFactory;
    exports.toFrequency = toFrequency;
    exports.toNote = toNote;
    exports.ɵa = MIDI_INPUT_QUERY;
    exports.ɵb = MIDI_OUTPUT_QUERY;

    Object.defineProperty(exports, '__esModule', { value: true });

})));

//# sourceMappingURL=ng-web-apis-midi.umd.js.map