import { from, fromEvent, merge } from 'rxjs';
import { filter, map, switchMap } from 'rxjs/operators';
import { NAVIGATOR } from '@ng-web-apis/common';
import { InjectionToken, inject } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @param {?} value
 * @param {?} min
 * @param {?} max
 * @return {?}
 */
function between(value, min, max) {
    return value >= min && value <= max;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Filter MIDI messages to aftertouch changes only
 * @return {?}
 */
function aftertouch() {
    return source => source.pipe(filter(({ data }) => between(data[0], 208, 223)));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Filter MIDI messages by channel
 *
 * @param {?} channel number from 0 to 15
 * @return {?}
 */
function filterByChannel(channel) {
    return source => source.pipe(filter(({ data }) => data[0] % 16 === channel));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Filter MIDI messages by MIDIInput id
 *
 * @param {?} id
 * @return {?}
 */
function filterById(id) {
    return source => source.pipe(filter(({ target }) => ((/** @type {?} */ (target))).id === id));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Filter MIDI messages by MIDIInput name
 *
 * @param {?} name
 * @return {?}
 */
function filterByName(name) {
    return source => source.pipe(filter(({ target }) => ((/** @type {?} */ (target))).name === name));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Filter MIDI messages to main volume changes only
 * @return {?}
 */
function mainVolume() {
    return source => source.pipe(filter(({ data }) => between(data[0], 176, 191) && data[1] === 7));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Filter MIDI messages to modulation wheel changes only
 * @return {?}
 */
function modulationWheel() {
    return source => source.pipe(filter(({ data }) => between(data[0], 176, 191) && data[1] === 1));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Filter MIDI messages to notes only
 *
 * IMPORTANT: It normalizes noteOff events to noteOn with 0 velocity
 * @return {?}
 */
function notes() {
    return source => source.pipe(filter(({ data }) => between(data[0], 128, 159)), map(event => {
        if (between(event.data[0], 128, 143)) {
            event.data[0] += 16;
            event.data[2] = 0;
        }
        return event;
    }));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Filter MIDI messages to pan changes only
 * @return {?}
 */
function pan() {
    return source => source.pipe(filter(({ data }) => between(data[0], 176, 191) && data[1] === 10));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Filter MIDI messages to pitch bend changes only
 * @return {?}
 */
function pitchBend() {
    return source => source.pipe(filter(({ data }) => between(data[0], 224, 239)));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Filter MIDI messages to polyphonic aftertouch changes only
 * @return {?}
 */
function polyphonicAftertouch() {
    return source => source.pipe(filter(({ data }) => between(data[0], 160, 175)));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Filter MIDI messages to program changes only
 * @return {?}
 */
function programChange() {
    return source => source.pipe(filter(({ data }) => between(data[0], 208, 223)));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Filter MIDI messages to sustain pedal changes only
 * @return {?}
 */
function sustainPedal() {
    return source => source.pipe(filter(({ data }) => between(data[0], 176, 191) && data[1] === 64));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Extract MIDI data from event
 * @return {?}
 */
function toData() {
    return source => source.pipe(map(({ data }) => data));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Extract data byte (2nd) from MIDI message
 *
 * NOTE: Some status messages do not have 2nd byte, use it when you're certain
 * @return {?}
 */
function toDataByte() {
    return source => source.pipe(map(({ data }) => data[1]));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Extract status byte (1st) from MIDI message
 * @return {?}
 */
function toStatusByte() {
    return source => source.pipe(map(({ data }) => data[0]));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Extract received time from MIDI event
 * @return {?}
 */
function toTimeStamp() {
    return source => source.pipe(map(({ timeStamp }) => timeStamp));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Extract value byte (3rd) from MIDI message
 *
 * NOTE: Some status messages do not have 3rd byte, use it when you're certain
 * @return {?}
 */
function toValueByte() {
    return source => source.pipe(map(({ data }) => data[2]));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const SYSEX = new InjectionToken('Require sysex MIDI access', {
    providedIn: 'root',
    factory: () => false,
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const MIDI_ACCESS = new InjectionToken('Promise for MIDIAccess object', {
    providedIn: 'root',
    factory: () => {
        /** @type {?} */
        const navigatorRef = inject(NAVIGATOR);
        /** @type {?} */
        const sysex = inject(SYSEX);
        return navigatorRef.requestMIDIAccess({ sysex });
    },
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const MIDI_INPUT = new InjectionToken('MIDIInput object');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const MIDI_MESSAGES = new InjectionToken('All incoming MIDI messages stream', {
    providedIn: 'root',
    factory: () => from(inject(MIDI_ACCESS)).pipe(switchMap(access => merge(...Array.from(access.inputs).map(([_, input]) => fromEvent((/** @type {?} */ (input)), 'midimessage'))))),
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const MIDI_OUTPUT = new InjectionToken('MIDIOutput object');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const MIDI_SUPPORT = new InjectionToken('Web MIDI API support', {
    factory: () => !!inject(NAVIGATOR).requestMIDIAccess,
});

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const MIDI_INPUT_QUERY = new InjectionToken('MIDIInput object id or name');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Provide MIDIInput by id
 *
 * @param {?} id
 * @return {?}
 */
function inputById(id) {
    return [
        {
            provide: MIDI_INPUT_QUERY,
            useValue: id,
        },
        {
            provide: MIDI_INPUT,
            deps: [MIDI_ACCESS, MIDI_INPUT_QUERY],
            useFactory: inputByIdFactory,
        },
    ];
}
/**
 * @param {?} midiAccess
 * @param {?} id
 * @return {?}
 */
function inputByIdFactory(midiAccess, id) {
    return midiAccess.then(access => access.inputs.get(id));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Provide MIDIInput by name
 *
 * @param {?} name
 * @return {?}
 */
function inputByName(name) {
    return [
        {
            provide: MIDI_INPUT_QUERY,
            useValue: name,
        },
        {
            provide: MIDI_INPUT,
            deps: [MIDI_ACCESS, MIDI_INPUT_QUERY],
            useFactory: inputByNameFactory,
        },
    ];
}
/**
 * @param {?} midiAccess
 * @param {?} name
 * @return {?}
 */
function inputByNameFactory(midiAccess, name) {
    return midiAccess.then(access => [...access.inputs.values()].find(input => input.name === name));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const MIDI_OUTPUT_QUERY = new InjectionToken('MIDIOutput object id or name');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Provide MIDIOutput by id
 *
 * @param {?} id
 * @return {?}
 */
function outputById(id) {
    return [
        {
            provide: MIDI_OUTPUT_QUERY,
            useValue: id,
        },
        {
            provide: MIDI_OUTPUT,
            deps: [MIDI_ACCESS, MIDI_OUTPUT_QUERY],
            useFactory: outputByIdFactory,
        },
    ];
}
/**
 * @param {?} midiAccess
 * @param {?} id
 * @return {?}
 */
function outputByIdFactory(midiAccess, id) {
    return midiAccess.then(access => access.outputs.get(id));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Provide MIDIOutput by name
 *
 * @param {?} name
 * @return {?}
 */
function outputByName(name) {
    return [
        {
            provide: MIDI_OUTPUT_QUERY,
            useValue: name,
        },
        {
            provide: MIDI_OUTPUT,
            deps: [MIDI_ACCESS, MIDI_OUTPUT_QUERY],
            useFactory: outputByNameFactory,
        },
    ];
}
/**
 * @param {?} midiAccess
 * @param {?} name
 * @return {?}
 */
function outputByNameFactory(midiAccess, name) {
    return midiAccess.then(access => [...access.outputs.values()].find(output => output.name === name));
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * Convert MIDI notes to frequencies
 *
 * @param {?} note MIDI note
 * @param {?=} tuning tuning for middle A (440 by default)
 * @return {?}
 */
function toFrequency(note, tuning = 440) {
    return Math.pow(2, (note - 69) / 12) * tuning;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @type {?} */
const COEFFICIENT = Math.pow(2, (1 / 12));
/**
 * Convert frequencies to MIDI notes
 *
 * @param {?} frequency
 * @param {?=} tuning tuning for middle A (440 by default)
 * @return {?}
 */
function toNote(frequency, tuning = 440) {
    return Math.round(Math.log(frequency / tuning) / Math.log(COEFFICIENT)) + 69;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { aftertouch, filterByChannel, filterById, filterByName, mainVolume, modulationWheel, notes, pan, pitchBend, polyphonicAftertouch, programChange, sustainPedal, toData, toDataByte, toStatusByte, toTimeStamp, toValueByte, MIDI_ACCESS, MIDI_INPUT, MIDI_MESSAGES, MIDI_OUTPUT, MIDI_SUPPORT, SYSEX, inputById, inputByIdFactory, inputByName, inputByNameFactory, outputById, outputByIdFactory, outputByName, outputByNameFactory, toFrequency, toNote, MIDI_INPUT_QUERY as ɵa, MIDI_OUTPUT_QUERY as ɵb };

//# sourceMappingURL=ng-web-apis-midi.js.map